//
//  YGViewController.m
//  Multipeer
//
//  Created by yagom on 13. 10. 16..
//  Copyright (c) 2013년 yagom. All rights reserved.
//

#import "YGViewController.h"
@import MultipeerConnectivity;

@interface YGViewController () <UITextFieldDelegate, MCBrowserViewControllerDelegate, MCSessionDelegate>

/// 메세지 내용을 입력할 필드
@property (nonatomic, assign) IBOutlet UITextField *chatField;

/// 대화 내용이 보여질 텍스트 뷰
@property (nonatomic, assign) IBOutlet UITextView *chatTextView;

/// 주변 기기를 찾을 때 사용할 기본 브라우저 컨트롤러
@property (nonatomic, strong) MCBrowserViewController *browserViewController;

/// 초대 수락등에 쓰이게 될 Advertiser assistant
@property (nonatomic, strong) MCAdvertiserAssistant *advertiser;

/// 나의 Peer ID
@property (nonatomic, strong) MCPeerID *peerID;

/// 내 세션
@property (nonatomic, strong) MCSession *session;

/// 주변 기기 찾기 버튼을 눌렀을 때
- (IBAction)clickFindButton:(id)sender;

/// 메세지 보내기 버튼을 눌렀을 때
- (IBAction)clickSendButton:(id)sender;

@end

@implementation YGViewController

#pragma mark - View Life Cycle

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    // 기기 이름으로 Peer ID 객체를 생성합니다.
    self.peerID = [[MCPeerID alloc] initWithDisplayName:[[UIDevice currentDevice] name]];
    
    // 위에서 생성한 Peer ID를 가지고 세션 객체를 생성합니다.
    self.session = [[MCSession alloc] initWithPeer:self.peerID];
    
    // Delegate를 설정해줍니다.
    [self.session setDelegate:self];
    
    // 주변 기기를 찾을 브라우져 뷰 컨트롤러 객체를 생성합니다.
    self.browserViewController = [[MCBrowserViewController alloc] initWithServiceType:@"message" session:self.session];
    
    // Delegate를 설정해줍니다.
    [self.browserViewController setDelegate:self];
    
    // Advertiser assistant 객체를 생성합니다.
    self.advertiser = [[MCAdvertiserAssistant alloc] initWithServiceType:@"message" discoveryInfo:nil session:self.session];
    
    // Advertiser assistant를 구동시킵니다.
    [self.advertiser start];
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#pragma mark - Button Method

// 주변기기 찾기 버튼을 눌렀을 때 호출됩니다.
- (IBAction)clickFindButton:(id)sender
{
    // 주변 기기를 찾을 수 있도록 브라우져 뷰 컨트롤러를 모달로 띄워줍니다.
    [self presentViewController:self.browserViewController animated:YES completion:NULL];
}

// SEND 버튼을 눌렀을 때 호출됩니다.
- (IBAction)clickSendButton:(id)sender
{
    [self.chatField resignFirstResponder];
    [self sendMessage];
}

#pragma mark - Send & Receive

- (void)sendMessage
{
    // 보낼 메세지입니다.
    NSString *message = self.chatField.text;
    
    // NSString 텍스트를 NSData로 변환합니다.
    NSData *messageData = [message dataUsingEncoding:NSUTF8StringEncoding];
    
    // 데이터 전달 중 에러를 받을 포인터.
    NSError *error = nil;
    
    // 현재 세션에 연결되어있는 Peer에게 데이터를 전달합니다.
    [self.session sendData:messageData toPeers:[self.session connectedPeers] withMode:MCSessionSendDataUnreliable error:&error];
    
    // 에러가 발생하지 않았다면
    if (error == nil)
    {
        // 채팅필드의 메세지를 없애고
        [self.chatField setText:nil];
        
        // 텍스트뷰에 메세지를 이어 붙여줍니다.
        [self receiveMessage:message fromPeer:self.peerID];
    }
    else // 에러가 발생했다면
    {
        // 에러를 로그로 봅니다.
        NSLog(@"%@", error);
    }
}

- (void)receiveMessage:(NSString *)message fromPeer:(MCPeerID *)peer
{
    // 결과적으로 보여질 텍스트입니다. \n는 줄바꿈입니다.
    NSString *allMessage = [self.chatTextView.text stringByAppendingFormat:@"%@ : %@\n\n", peer.displayName, message];
    
    // 텍스트뷰에 메세지를 세팅합니다. 화면 세팅은 메인스레드에서 진행되어야 합니다.
    [self.chatTextView performSelectorOnMainThread:@selector(setText:) withObject:allMessage waitUntilDone:YES];
}


#pragma mark - MCBrowserViewControllerDelegate

// 브라우저 뷰 컨트롤러의 Done 버튼을 눌렀을 때 호출됩니다.
- (void)browserViewControllerDidFinish:(MCBrowserViewController *)browserViewController
{
    [self.browserViewController dismissViewControllerAnimated:YES completion:NULL];
}

// 브라우저 뷰 컨트롤러의 Cancel 버튼을 눌렀을 때 호출됩니다.
- (void)browserViewControllerWasCancelled:(MCBrowserViewController *)browserViewController
{
    [self.browserViewController dismissViewControllerAnimated:YES completion:NULL];
}

#pragma mark - UITextFieldDelegate

// 메세지 입력시 retrun 버튼을 눌렀을 때 호출됩니다.
- (BOOL)textFieldShouldReturn:(UITextField *)textField
{
    [textField resignFirstResponder];
    [self sendMessage];
    return YES;
}

#pragma mark - MCSessionDelegate

// 상대방의 연결상태가 변경되었을 때 호출되는 델리게이트 메소드입니다.
- (void)session:(MCSession *)session peer:(MCPeerID *)peerID didChangeState:(MCSessionState)state
{
    switch (state)
    {
        case MCSessionStateConnected:
            NSLog(@"Connected!");
            break;
            
        case MCSessionStateConnecting:
            NSLog(@"Connecting!");
            break;
            
        case MCSessionStateNotConnected:
            NSLog(@"Not Connected!");
            break;
            
        default:
            break;
    }
}

// 상대방으로부터 데이터를 받았을 때 호출되는 델리게이트 메소드입니다.
- (void)session:(MCSession *)session didReceiveData:(NSData *)data fromPeer:(MCPeerID *)peerID
{
    // 수신한 NSData 데이터를 NSString 객체로 변환합니다.
    NSString *message = [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding];
    
    // 수신한 메세지를 텍스트뷰에 표시해 줍니다.
    [self receiveMessage:message fromPeer:peerID];
    
}

// 상대방으로부터 스트리밍 데이터를 받을때 호출되는 델리게이트 메소드입니다.
- (void)session:(MCSession *)session didReceiveStream:(NSInputStream *)stream withName:(NSString *)streamName fromPeer:(MCPeerID *)peerID{}

// 상대방으로부터 리소스를 받을때 호출되는 델리게이트 메소드입니다.
- (void)session:(MCSession *)session didStartReceivingResourceWithName:(NSString *)resourceName fromPeer:(MCPeerID *)peerID withProgress:(NSProgress *)progress{}

// 리소스 수신이 완료되었을 때 호출되는 델리게이트 메소드입니다.
- (void)session:(MCSession *)session didFinishReceivingResourceWithName:(NSString *)resourceName fromPeer:(MCPeerID *)peerID atURL:(NSURL *)localURL withError:(NSError *)error{}

@end
